/*
 * TreeFromRawParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <sax/FromXMLParserHelper.h>
#include <exception/CommonException.h>
#include <tree/Tree.h>
#include <tree/TreeClasses.h>
#include <alphabet/Symbol.h>
#include <alphabet/BarSymbol.h>
#include "TreeFromRawParser.h"

namespace tree {

Tree TreeFromRawParser::parseTree(std::deque<sax::Token>::iterator& input) const {
	return parseTree(input, std::set<FEATURES>({FEATURES::RANKED_TREE, FEATURES::PREFIX_RANKED_TREE, FEATURES::PREFIX_RANKED_BAR_TREE, FEATURES::UNRANKED_TREE}));
}

Tree TreeFromRawParser::parseTree(std::deque<sax::Token>::iterator& input, const std::set<FEATURES>& features) const {
	UnrankedTree tree(this->parseContent(input));

	if(features.count(FEATURES::UNRANKED_TREE)) return Tree{tree};

	if(features.count(FEATURES::PREFIX_RANKED_TREE)) return Tree{PrefixRankedTree{RankedTree{tree}}};

	if(features.count(FEATURES::PREFIX_RANKED_BAR_TREE)) return Tree{PrefixRankedBarTree{alphabet::Symbol{alphabet::BarSymbol{}}, RankedTree{tree}}};

	if(features.count(FEATURES::RANKED_TREE)) return Tree{RankedTree{tree}};

	throw exception::CommonException("Invalid input");
}

std::tree < alphabet::Symbol > TreeFromRawParser::parseContent(std::deque<sax::Token>::iterator& input) const {
	if (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		std::tree < alphabet::Symbol > node ( alphabet::symbolFrom(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::START_ELEMENT)), {});
		while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT) || sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::CHARACTER)) {
			node.push_back(node.root( ), this->parseContent(input));
		}
		sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::END_ELEMENT);
		return node;
	} else if(sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::CHARACTER)) {
		return std::tree < alphabet::Symbol > (alphabet::symbolFrom(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER)), {});
	} else {
		throw exception::CommonException("Invalid token stream");
	}
}

} /* namespace tree */
